/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <errors.h>
#include <storage.h>
#include <io.h>
#include <steen.h>
#include <module.h>
#include <structur.h>
#include <util.h>

char Status_fmt[] = "%10d";   /* format of status in files Im and Ker */
char Diff_hdr[] = "%10d %10d";   /* format of header in Diff files    */


main(argc,argv) int argc; char *argv[];
{
int i,j,nread,numops,deg,dim,found,status,g,gen,op,
    lgen,lop,                /* leading gen and op                     */
    s,t,                     /* bidegree to do                         */
    seqno,                   /* seqno of op in c                       */
    degree_done,             /* of Diff.s-1 then of Diff.s             */
    dim_codomain,            /* dim over A of codomain                 */
    image_dim,               /* dim over A of codomain in degree < t   */
    **image_gen,**image_op,  /* (gen,op) for corresp image entry       */
    next_gen,                /* next gen in filt s                     */
    old_next_gen,            /* save original value of next_gen        */
    *degree;                 /* internal degree of gens of C.s-1       */
Element *e, *savec;          /* image elements                         */
/* ------------------------------------------------------------------- */

Element  ***image_x,
         ***image_dx;
                             /* These are each arrays of arrays of     */
                             /* pointers to Elements, such that the    */
                             /* non-NULL image_dx[g][o] are a basis    */
                             /* for the image, and                     */
                             /*  d(image_x[g][o]) = image_dx[g][o]     */
                             /* Further, image_dx[g][o] = o * g +      */
                             /*  higher terms                          */

/* ------------------------------------------------------------------  */
TERM **c;                    /* c is in Ker(s-1,t)                     */
FILE *Diff,                  /* Diff.s-1 and then Diff.s               */
     *Ker,                   /* Ker_s.t                                */
     *Im,                    /* Im_s.t                                 */
     *OIm;                   /* OIm_s.t                                */
char filename[20],message[100];


/* -------------- check calling sequence -----------------------------  */
if (argc != 3) {
  printf("\nWrong number %d of args  Should be 2.\n",argc-1);
  for (i=0;i<argc;i++) fprintf(stderr,"%3d: %s\n",i,argv[i]);
  exit(1);
 }
/* ------------------------------ get s,t ------------------------------ */

sscanf(argv[1],"%d",&s);
sscanf(argv[2],"%d",&t);

/* ----------- process Diff.s-1   -------------------------------------  */
/* Check previous filtration (or module) for completness, get dimension  */
/*  and degrees of gens to compute A dim of the codomain                 */

if (s > 0) {
   sprintf(filename,"Diff.%d",s-1);
   if ((Diff = fopen(filename,"r")) == NULL) {
      sprintf(message,"\nFile not found: %s\n",filename);
      terminate(message,BAD_DIFF);
     }

   if ((nread = fscanf(Diff,Diff_hdr,&dim_codomain,&degree_done)) != 2) {
      sprintf(message,"\nBad %s: read only %d variables from header.",
                     filename,nread);
      terminate(message,BAD_DIFF);
     }

   if (degree_done < t) {
      sprintf(message,"\n%s incomplete.  Done through degree %d",
                     filename,degree_done);
      terminate(message,BAD_DIFF);
     }
/* read Diff.s-1 to get degrees of generators so we'll know how much   */
/*  space to allocate                                                  */

   degree = (int *)malloc(dim_codomain*sizeof(int));
   image_dim = 0;
   for (g=0; g<dim_codomain; g++) {
      fscanf(Diff,"%d",&degree[g]);
      skip_Element(Diff);
      if (degree[g] <= t) image_dim = 1 + g;
   }

   fclose(Diff);
                  /* done:  have dim_codomain, image_dim, and degrees */
}
                  /* s = 0   -    get these from the module instead   */
                  /*     (except dim_codomain, which is not needed)   */
else
   image_dim = mod_basis(t,&degree);

c = (TERM **) malloc(image_dim * sizeof(TERM *));
image_dx = (Element ***) malloc(image_dim * sizeof(Element **));
image_x = (Element ***) malloc(image_dim * sizeof(Element **));
image_gen = (int **) malloc(image_dim * sizeof(int *));
image_op = (int **) malloc(image_dim * sizeof(int *));


/* allocate space for c's terms now, since we know their degrees.     */
/* allocate entries in image and set them to NULL                      */

for (g=0; g<image_dim; g++)
{  deg = t - degree[g];
   numops = dimA(deg);
                   
   if (numops )
   { c[g] = get_TERM(numops);
     c[g]->gen = g;
     c[g]->deg = deg;
     image_dx[g] = (Element **)malloc(numops * sizeof(Element *));
     image_x[g] = (Element **)malloc(numops * sizeof(Element *));
     image_gen[g] = (int *) malloc (numops * sizeof(int));
     image_op[g] = (int *) malloc (numops * sizeof(int));
     for (i=0; i<numops; i++) 
	  image_dx[g][i] = image_x[g][i] = NULL;
   }
   else    /* numops == 0  */
   { image_dx[g] = NULL;
     c[g] = NULL;
   }
}

/* --------------------- process Diff.s   ---------------------------  */
/* Check this filtration for completeness, and get the number of gens  */
/*  (= number of next gen).  Open for reading and writing since we     */
/*  will update the differential file for this filtration.             */
/* If ok, seek to the end in preparation for adding new generators.    */

sprintf(filename,"Diff.%d",s);
if ((Diff = fopen(filename,"r+")) == NULL) {
   sprintf(message,"\nFile not found: %s\n",filename);
   terminate(message,BAD_DIFF);
}

if ((nread = fscanf(Diff,Diff_hdr,&next_gen,&degree_done)) != 2) {
   sprintf(message,"\nBad %s: read only %d variables from header.",
                  filename,nread);
   terminate(message,BAD_DIFF);
}
if (degree_done < t - 1) {
   sprintf(message,"\n%s incomplete.  Done through degree %d",
                  filename,degree_done);
   terminate(message,BAD_DIFF);
}
if (degree_done > t - 1) {
   fprintf(stderr,"\n%s already complete through degree %d",
                  filename,degree_done);
   fprintf(stderr,"\nContinuing anyway.");
}
old_next_gen = next_gen;
fseek(Diff,0L,2);

/* -------------  open Ker_s-1.t  ----------------------------------   */
/* ----- Differential files are ok.  Check kernel status.  Must open   */
/*  for reading and writing so we can update he status code.           */

sprintf(filename,"Ker_%d.%d",s-1,t);
if ((Ker = fopen(filename,"r")) == NULL) {
     sprintf(message,"\nError opening %s ",filename);
     terminate(message,BAD_KER);
}
fscanf(Ker,Status_fmt,&status);
if (status != 1)
{    sprintf(message,"\n%s is incorrect: status = %d\n",filename,status);
     terminate(message,BAD_KER);
}

/* Kernel is ok.  Open OIm now so if there is a problem we'll quit now  */
/*  instead of wasting our effort .                                     */

sprintf(filename,"OIm_%d.%d",s,t);
if ((OIm = fopen(filename,"w")) == NULL) {
     sprintf(message,"\nError opening %s ",filename);
     terminate(message,BAD_IM);
}

/* OIm opened ok.  Check and read Im_s.t -----------------------------  */

sprintf(filename,"Im_%d.%d",s,t);
if ((Im = fopen(filename,"r")) == NULL) {
     sprintf(message,"\nError opening %s ",filename);
     terminate(message,BAD_IM);
}
fscanf(Im,Status_fmt,&status);
if (status != 1)
{    sprintf(message,"\n%s is incorrect: status = %d\n",filename,status);
     terminate(message,BAD_IM);
}

while (fscanf(Im,"%d %d",&gen,&op) != EOF)
{ e = read_Element(Im);
  lgen = e->item[0]->gen;
  lop = first_seqno_in(e->item[0]);
  image_dx[lgen][lop] = e;
  image_x[lgen][lop] = read_Element(Im);
  image_gen[lgen][lop] = gen;
  image_op[lgen][lop] = op;
}
fclose(Im);

/* Image now constructed.  Start scanning kernel                       */

while (fscanf(Ker,"%d %d",&gen,&op) != EOF)
{ for (i=0; i<image_dim; i++) if (c[i] != NULL) zero_op(c[i]);
  if (read_exp_Element(Ker,c,image_dim) == EOF)
   { sprintf(message,"\nPremature EOF on kernel. gen,op = %d,%d\n",gen,op);
     terminate(message,BAD_KER);
   }
/* save the original kernel element, to be used as the target of a new
   generator if it is not in the image thus far.  This produces cleaner
   relations, it seems, than the old method of hitting the row reduced
   version of this, which has leading term distinct from all leading terms
   seen thus far, but has been complicated in the process.              */

savec = condense(c,image_dim);

/*  Now reduce against the image so far.                               */


for (found=i=0; (!found) & (i < image_dim); i++)
for (seqno = first_seqno_in(c[i]);
      (!found) & (seqno >= 0);
        seqno = next_seqno_in(c[i],seqno) )
if ((e = image_dx[i][seqno]) != NULL)      /* cancel off leading term  */
{   for (j=0; j < e->len; j++)
      if (subtract(c[e->item[j]->gen],e->item[j]))
      {  fclose(Im); fclose(Ker); fclose(Diff);
         fprintf(stderr,
	    "\nError subtracting item %d of image from c. c is:\n",j);
         write_exp_Element(stderr,c,'s',image_dim);
         fprintf(stderr,"\n and image[%d][%d] is:\n",i,seqno);
         write_Element(stderr,e,'s');
         terminate(" ",BAD_REDUCE);
       }
}
else        /*  not NULL so c has a new leading term.  Add a new gen.  */
  { image_dx[i][seqno] = condense(c,image_dim);
    image_gen[i][seqno] = next_gen;
    image_op[i][seqno] = 0;
    image_x[i][seqno] = get_Element(1);
    image_x[i][seqno]->item[0] = get_TERM(1);
    image_x[i][seqno]->item[0]->gen = next_gen++;
    image_x[i][seqno]->item[0]->deg = 0;
    image_x[i][seqno]->item[0]->op[0] = 0x80;

    fprintf(Diff,"\n%d\n",t);
    write_Element(Diff,savec,'c');
/*  used to write out image_dx[i][seqno] instead of savec              */

    found = 1;
  }
/* If we fall out the end of the loop, without setting found = 1, c    */
/* should be reduced to 0. Check to be sure that is true.              */

if (!found)
{
 for (i=0; i < image_dim; i++) if (is_not_zero(c[i]))
  { fprintf(stderr,
            "\nc is still nonzero after falling thru reduce loop. c:");
    write_exp_Element(stderr,c,'s',image_dim);
    terminate(" ",BAD_REDUCE);
  }

/* Ok: c really is zero. --------------------------------------------  */

} /* end if !found  ----------------------------------------------     */

} /* end while read Ker.  We're done.  Set status codes and quit.      */

fprintf(stderr,"\n%d new generators were added in bidegree (%d,%d).\n",
	       next_gen-old_next_gen,s,t);
if ((degree_done > t-1) && (old_next_gen < next_gen))
  fprintf(stderr,
          "Possible error: Diff claimed to be complete through degree %d\n",
          degree_done);
if (degree_done == t-1)
{ rewind(Diff);
  fprintf(Diff,Diff_hdr,next_gen,t);
}
else if (old_next_gen < next_gen)
{ rewind(Diff);
  fprintf(Diff,Diff_hdr,next_gen,degree_done);
}
fclose(Diff);

rewind(Ker);
fprintf(Ker,Status_fmt,2);
fclose(Ker);

/* Now write out the Image in order of leading gen and op for speedier */
/* access by other programs.                                           */

/* Scan image and write it in order of leading terms                   */

for (lgen=0; lgen<image_dim; lgen++)
  if (image_dx[lgen] != NULL)
    for (lop=0,numops = dimA(t-degree[lgen]);
	  lop < numops; lop++)
      if (image_dx[lgen][lop] != NULL)
       { fprintf(OIm,"%d %d %d %d\n",
	   lgen,lop,image_gen[lgen][lop],image_op[lgen][lop]);
	 write_Element(OIm,image_dx[lgen][lop],'c');
	 write_Element(OIm,image_x[lgen][lop],'c');
	 fprintf(OIm,"\n");
       }


fclose(OIm);
	


}

