/* -------------------------------------------------------------------	*/
/* Bob, Stole your code from sortDef amd others for this --- Tyler	*/
/* -------------------------------------------------------------------- */

/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
/*#include <malloc.h> */
#include <stdlib.h>

#include <errors.h>

/* --------------------------------------------------------------- */

int main(argc,argv) int argc; char *argv[];

{
int i, j, k, m, n, sgen, dgen, op, count;
int dim, maxdeg, matrixdim;
int *degrees;
int ***matrix;
int ***output;
int **active;

FILE *In, *Out;

/* -------------- check calling sequence -----------------------------  */
if (argc != 3)
{
  fprintf(stderr,
    "\nUsage: %s <Def> <newDef>  \n",argv[0]);
  fprintf(stderr,
    "  Produces a new module definition dual to <Def> and writes it in\n");
  fprintf(stderr,
    "  the file <newDef>.\n");
  exit(1);
}

/* ------------------ open files --------------------------------------- */

if ((In = fopen(argv[1],"r")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to read\n",argv[1]);
  exit(1);
}

if ((Out = fopen(argv[2],"w")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to write\n",argv[2]);
  exit(1);
}

/* ------------------ Now get headers  ------------------------------ */

fscanf(In,"%d",&dim);
fprintf(Out,"%d\n",dim);

/* ------------------------ Load old degrees ----------------------- */

degrees = (int *)malloc(dim*sizeof(int));
for (i=0; i < dim; i++)
{
  fscanf(In,"%d",&degrees[i]);
}
maxdeg = degrees[dim-1] - degrees[0] + 1; // contains an identity Sq^0 page

matrix = (int ***)malloc(dim * sizeof(int **));
output = (int ***)malloc(dim * sizeof(int **));
active = (int **)malloc(dim * sizeof(int *));
for(i=0;i<dim;i++)
{
  matrix[i] = (int **)malloc(dim * sizeof(int *));
  output[i] = (int **)malloc(dim * sizeof(int *));
  for(j=0;j<dim;j++)
  {
    matrix[i][j] = (int *)malloc(maxdeg * sizeof(int));
    output[i][j] = (int *)malloc(maxdeg * sizeof(int));
    active[j] = (int *)malloc(maxdeg * sizeof(int));
    for(k=0;k<maxdeg;k++)
    {
      matrix[i][j][k] = 0;
      output[i][j][k] = 0;
      active[j][k] = 0;
    }
  }
  matrix[i][i][0] = 1;
  output[i][i][0] = 1;
}

for (i=dim-1; i >= 0; i--)
{
  fprintf(Out,"%d ",-degrees[i]);
}

fprintf(Out,"\n\n");

/* ---------------------- Read in the old action ------------------- */


while (EOF != fscanf(In, "%d %d %d", &sgen, &op, &count))
{
  if ((sgen < 0 ) || (sgen >= dim))
  { fprintf(stderr,
      "Operation on nonexistent generator %d\n\n",sgen);
    exit(1);
  } else
  if (count < 1) 
  { fprintf(stderr,
      "Bad count: %d\n\n",count);
     exit(1);
  } else
  if (op < 1) 
  { fprintf(stderr,
      "Bad Steenrod operation: %d\n\n",op);
     exit(1);
  } else
    for(i=0;i<count;i++)      
    { fscanf(In, "%d", &dgen);  
      if ((dgen < 0 ) || (dgen >= dim))
      { fprintf(stderr,
          "Operation result nonexistent: %d\n\n",dgen);
        exit(1);
      } 
      if (op < 1) 
      { fprintf(stderr,
          "Bad Steenrod operation: %d\n\n",op);
        exit(1);
      }
      if (op + degrees[sgen] != degrees[dgen])
      { fprintf(stderr,
          "Degree error: Sq(%d)(%d) contains %d, ",op,sgen,dgen);
	fprintf(stderr,
           "but deg(%d) = %d while deg(%d) = %d\n",sgen,degrees[sgen],
		dgen,degrees[dgen]);
      } else
      matrix[sgen][dgen][op] = 1;
    }
}

/* ------------------ Dualize the action by iteration -------------- */
 for(k=1;k<maxdeg;k++) // one op at a time
   for (i=0;i<dim;i++) // go over all matrix entries
     for(j=0;j<dim;j++)
     {
       for(m=0;m<k;m++) // recurse over lower ops w/ Cartan formula
	 for(n=0;n<dim;n++) // matrix multiplication index
         {
	   output[i][j][k] ^= ((output[n][j][m]) & (matrix[i][n][k-m]));
         }
       if (output[i][j][k])
	 active[j][k]++;
     }

/* ---------------- Print out the resulting definition ------------- */
  for(j=dim-1;j>=0;j--)
    for(k=0;k<maxdeg;k++)
    {
      if (active[j][k])
      {
	fprintf(Out,"%d %d %d ",dim-1-j,k,active[j][k]);
	for(i=0;i<dim;i++)
	  if(output[i][j][k])
          {
	    if (!((i+1) & 15)) fprintf(Out,"\n");
            fprintf(Out,"%d ",dim-1-i);
	  }
        fprintf(Out,"\n");
      }
    }

/* --------------------------- Done   ------------------------------ */

fclose(In);
fclose(Out);
return(0);
   
}

