/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */


#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <storage.h>
#include <io.h>
#include <steen.h>
#include <structur.h>
#include <tensor.h>
#include <util.h>

main(argc,argv) int argc; char *argv[];
{
int s1,g1,		/* compute map(g1) by				*/
    s,t,		/*    lifting over the image of C(s,t)		*/
    lgen,lop,gen,op,	/* used to index image				*/
    numops,i,j,g,	/* indices and limits				*/
    len,deg,		/* used to construct exp_Elements		*/
    dimCod,		/* A dim of module containing map(dg)'s		*/
    *degree,		/*  degrees of those A generators		*/
    seqno;		

TERM   **mapg1,		/* map(g1)					*/
       **dmapg1;	/* map(dg1)					*/
Element *e;		/* image elements being subtracted		*/

Tensor_shape *shape;	/* easy way to keep track of gens and degrees	*/

FILE *Work,		/* work file					*/
     *OIm,		/* Ordered Image file				*/
     *Map;		/* Map file					*/


Element  ***image_x,
         ***image_dx;
                             /* These are each arrays of arrays of     */
                             /* pointers to Elements, such that the    */
                             /* non-NULL image_dx[g][o] are a basis    */
                             /* for the image, and                     */
                             /*  d(image_x[g][o]) = image_dx[g][o]     */
                             /* Further, image_dx[g][o] = o * g +      */
                             /*  higher terms                          */

char filename[100],
     message[100];

/* -------------- check calling sequence -----------------------------  */
if (argc != 4)
{
  printf("\nUsage: liftmap s t workfileprefix\n");
  exit(1);
}

/* -------------- get s and t ------------------------------------	*/

sscanf(argv[1],"%d",&s);
sscanf(argv[2],"%d",&t);

/* -------------- get shape of Codomain --------------------------	*/
shape = read_shape(s);

/* ------------------------------------------------------------------	*/
/*  Get A dimension and degrees of generators for module OIm sits in.	*/
/*  This is Codomain[s-1] if s > 0, Module if s=0.			*/
/*  N.B.  It is CRUCIAL that liftmap be compiled using the module.o	*/
/*        for the module in whose resolution it is doing the lifts.	*/

if (s == 0) dimCod = mod_basis(t,&degree);
else { dimCod = shape->numgen[s-1];
       degree = shape->deg[s-1];
     }

/* -------------- open Work file ------------------------------------	*/

sprintf(filename,"%s%d.%d",argv[3],s,t);
if ((Work = fopen(filename,"r")) == NULL)
{  sprintf(message,"\nCannot open work file: %s\n",filename);
terminate(message,BAD_WORKFILE);
}


/* -------------- construct Image of C(s,t) -------------------------	*/
       
image_x = (Element ***) malloc(dimCod * sizeof(Element **));
image_dx = (Element ***) malloc(dimCod * sizeof(Element **));

/* allocate entries in image and set them to NULL                      */

for (g=0; g < dimCod; g++)
{  
   numops = dimA(t - degree[g]);
                   
   if (numops )
   { 
     image_x[g] = (Element **) malloc(numops * sizeof(Element *));
     image_dx[g] = (Element **) malloc(numops * sizeof(Element *));
     for (i=0; i<numops; i++) 
        image_x[g][i] = image_dx[g][i] = NULL;
   }
   else    /* numops == 0  */
     image_x[g] = image_dx[g] = NULL;
}


/* Now read OIm and file it by leadgen and leadop			*/

sprintf(filename,"OIm_%d.%d",s,t);
if ((OIm = fopen(filename,"r")) == NULL) {
     sprintf(message,"\nError opening %s \n",filename);
     terminate(message,BAD_IM);
}

while (fscanf(OIm,"%d %d %d %d",&lgen,&lop,&gen,&op) != EOF)
{
  image_dx[lgen][lop] = read_Element(OIm);
  image_x[lgen][lop] = read_Element(OIm);
}

fclose(OIm);

/* --------------- create dmapg1 and mapg1 -----------------------------	*/

dmapg1 = (TERM **) malloc(dimCod * sizeof(TERM *));
for (g=0; g < dimCod; g++) dmapg1[g] = NULL;

mapg1 = (TERM **) malloc(shape->numgen[s] * sizeof(TERM *));
for (g=0; g < shape->numgen[s]; g++)
  if ((len = dimA(deg = t - shape->deg[s][g])) > 0)
  {  mapg1[g] = get_TERM(len);
     mapg1[g]->gen = g;
     mapg1[g]->deg = deg;
  }
  else mapg1[g] = NULL;

/*  ---------------- Now process work file: ----------------------	*/
/*   read each element, lift it, and enter the result into the		*/
/*   appropriate Map file.						*/

while (EOF != fscanf(Work,"%s %d %d",filename,&s1,&g1))
{  
  read_exp_Element(Work,dmapg1,dimCod);
  for (g=0; g < shape->numgen[s]; g++) 
    if (mapg1[g] != NULL) zero_op(mapg1[g]);

  for (i=0; i < dimCod; i++)
      for (seqno = first_seqno_in(dmapg1[i]);
	    seqno >= 0;
	      seqno = next_seqno_in(dmapg1[i],seqno))
      { if ((e = image_dx[i][seqno]) == NULL)
	 { sprintf(message,"\nCannot lift leading term %d,%d\n",
		  i,seqno);
	   terminate(message,BAD_LIFT);
	 }
	for (j=0; j < e->len; j++)
	{ if (dmapg1[g = e->item[j]->gen] == NULL) 
           if ((len = dimA(deg = t - degree[g])) > 0)
             {  dmapg1[g] = get_TERM(len);
                dmapg1[g]->gen = g;
                dmapg1[g]->deg = deg;
		zero_op(dmapg1[g]);
              }
	  if (subtract(dmapg1[e->item[j]->gen],e->item[j]))
		terminate("\nError in subtraction.\n",BAD_REDUCE);
	}

        if ((e = image_x[i][seqno]) != NULL)
	for (j=0; j < e->len; j++)
	  if (subtract(mapg1[e->item[j]->gen],e->item[j]))
		terminate("\nError in subtraction.\n",BAD_REDUCE);
      }

/*  It's reduced. Now check that the result is really zero		*/

   for (i=0; i < dimCod; i++) 
   if (is_not_zero(dmapg1[i]))
    { fprintf(stderr,
       "\ndmapg1 is still nonzero after falling thru reduce loop. dmapg1:");
      write_exp_Element(stderr,dmapg1,'s',dimCod);
      fprintf(stderr,"\nfilename,s1,g1 are: %s %d %d\n");
      terminate(" ",BAD_REDUCE);
    }

/* Ok: dmapg1 really is zero. --------------------------------------------  */

  return_TERMs(dmapg1,dimCod);

  if ((Map = fopen(filename,"a")) == NULL)
  {  sprintf(message,"\nCannot open file  %s to append.\n",filename);
     terminate(message,BAD_MAP);
  }
  fprintf(Map,"\n%d %d\n",s1,g1);
  write_exp_Element(Map,mapg1,'c',shape->numgen[s]);
  fclose(Map);
  
 }
exit(0);
}
