/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#ifndef STRUC_CONTROL

typedef struct { unsigned    len;      /* number of bits in op      */
                 unsigned    gen;      /* space to insert gen that  */
                                       /*          this op acts on  */
               int           deg;      /* internal degree of op     */
               unsigned char op[1];    /* bitmask                   */
                 } TERM;

/* -------------------------------------------------------------------  */
/* op is actually going to be     char  op[d]                           */
/*                                                                      */
/* where  d = ceiling of len/8, when it is allocated.                   */
/*                                                                      */
/* Also, len = dimA(deg) is redundant, but we don't want to have to     */
/* constantly recalculate such a basic item.                            */
/* -------------------------------------------------------------------  */

typedef struct { unsigned    len;          /* number of TERMs           */
                     TERM   *item[1];
               }  Element;

/* -------------------------------------------------------------------  */
/* This is the compact (sparse case) representation of an element.      */
/* The expanded form is simply TERM *val[dim], where dim = dim over A   */
/* of the relevant module.                                              */
/* -------------------------------------------------------------------  */

/* -------------------------------------------------------------------  */
/* Tensors and their TTERMs                                             */
/* -------------------------------------------------------------------  */

typedef struct { int		filt;
		 int		deg;
		 int		opdeg;
		 unsigned	gen;
		 unsigned 	seq;
		 Element 	*RHS;
                 } TTERM;

/* -------------------------------------------------------------------  */
/* The TTERM represents op * gen (x) RHS, where                         */
/*   degree(op) = opdeg, seqno(op) = seq,                               */
/*   homological degree of gen = filt, and                              */
/*   internal degree of the LHS (op * gen) = deg, so that               */
/*   deg(gen) = deg - opdeg.                                            */
/* -------------------------------------------------------------------  */

typedef struct { unsigned    len;          /* number of TTERMs          */
                    TTERM   *item[1];
               }  Tensor;

/* -------------------------------------------------------------------  */
/* This is the compact (sparse case) representation of a tensor.        */
/* The expanded form is Element *****T, where T[s1][g][opdeg][seq] is   */
/* the RHS of the terms with LHS equal to op*g, for g in filt s1,       */
/* and op of degree opdeg and sequence number seq                       */
/*                                                                      */
/* Note that                                                            */
/*   T   may be NULL, indicating the zero tensor                        */
/*   T[s1]   may be NULL, meaning no terms with LHS of filt s1          */
/*   T[s1][g]   may be NULL, meaning no terms with LHS of filt s1 and   */
/*                      gen g                                           */
/*   T[s1][g][opdeg]   may be NULL, meaning no terms with LHS gen g of  */
/*                       filt s1 and an op of degree opdeg              */
/*   T[s1][g][opdeg][seq] may be NULL, meaning that the corresponding   */
/*                               RHS is 0.                              */
/* To describe the sizes of each subarray of an expanded tensor, we     */
/*   use the structure below (Tensor_shape).                            */
/* -------------------------------------------------------------------  */

typedef struct { int	s;
		 int	t;
		 int *numgen;
		 int	**deg;
	       } Tensor_shape;

/* ----------------------------------------------------------- */
/* This describes a tensor of degree (s,t), whose LHS can      */
/*   therefore be of any filt from 0 to s and any internal     */
/*   degree from 0 to t					       */
/* ----------------------------------------------------------- */
/* Note that numgen and deg can point to arrays of a size      */
/*  appropriate to the largest possible filtration s, and by   */
/*  changing the value of s we can restrict attention to the   */
/*  part we want, and keep the size of resulting tensors down. */
/* ----------------------------------------------------------- */
/* The order s,g,opdeg,seq is chosen on the grounds that:      */
/*   the range of values possible for seq depends on all the   */
/*     other values, so seq must be last,                      */
/*   the range of values of g depends on s so g must follow s, */
/*   the coproduct applied to op*g will produce terms          */
/*      containing g and many different opdegs, so             */
/*   on the general principle that the variables with the      */
/*     smallest range of values should come first,  g should   */
/*     come before t.                                          */
/* ----------------------------------------------------------- */


#define STRUC_CONTROL      to avoid rereading this
#endif

