/* -------------------------------------------------------------------	*/
/* Copyright (C) 2008 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.8								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
/*#include <malloc.h> */
#include <stdlib.h>

#include <errors.h>
/* #include <util.h>
   #include <steen.h>
   #include <structur.h>
   #include <storage.h>
   #include <io.h>
*/

typedef struct module_element {
			       int first;
	     struct module_element *next;} module_element;


/* ------------------------------------------------------------------ */
module_element * insert_gen(g,e)  int g; module_element *e;
{
  if ((e==NULL) || g < e->first)
  {
    module_element *e1=(module_element *)malloc(sizeof(module_element));

    e1->first=g;
    e1->next=e;
    return(e1);
  }
  if (g == e->first)  /*bad data file */
  {
    fprintf(stderr,"bad data file: Def\n");
    return (e->next);
  }
  e->next=insert_gen(g,e->next);
  return(e);
}

/* ------------------------------------------------------------------ */
module_element *** read_sqs(file,numgen,degrees,name) 
FILE *file;
int numgen;
int *degrees;
char *name;
{ 
  int maxdeg, i, gen, op, count, g;
  module_element ***sq_value;

  for (maxdeg=degrees[0],gen=1;gen<numgen;gen++)
    if (maxdeg < degrees[gen]) maxdeg = degrees[gen];

  sq_value=(module_element ***)malloc(numgen*sizeof(module_element **));
  for (gen=0;gen<numgen;gen++)
  {
    sq_value[gen]=(module_element **)malloc(
	 (1+maxdeg - degrees[gen])*sizeof(module_element *));
    sq_value[gen][0] = (module_element *)malloc(sizeof(module_element));
    sq_value[gen][0]->first = gen;
    sq_value[gen][0]->next = NULL;
    for(i=1;i<=maxdeg - degrees[gen];i++)
      sq_value[gen][i]=NULL;
  }
    		/* sq_value[gen][op]'s are initially NULL 	*/
		/* except Sq(0) is the identity op  		*/

  while (EOF != fscanf(file, "%d %d %d", &gen,&op,&count))
    for(i=0;i<count;i++)      
    { fscanf(file, "%d", &g);  
      if (op + degrees[gen] != degrees[g])
      { fprintf(stderr,
          "Degree error in %s: Sq(%d)(%d) contains %d\n ",
           name,op,gen,g);
	fprintf(stderr,
          "  but deg(%d) = %d while deg(%d) = %d\n",
              gen,degrees[gen],g,degrees[g]);
      }
      sq_value[gen][op]=insert_gen(g,sq_value[gen][op]);
    }
fclose(file);
return(sq_value);
}


/* --------------------------------------------------------------- */

int main(argc,argv) int argc; char *argv[];

{

int g,i,j,k,n,l,maxdeg, maxdeg1, maxdeg2;
int x1,x2,lo,hi,d,d1,mindeg, mindeg1, mindeg2;
int dim1, dim2, dim;
int *deg1, *deg2, *deg;
int  **product, *first, *second;
int  *first1, *second1;
int  *first2, *second2;
module_element *s, *g1, *g2;
module_element ***sq1;
module_element ***sq2;
module_element ***sq=NULL;

FILE *In1, *In2, *Out;
char message[100];

/* -------------- check calling sequence -----------------------------  */
if ((argc != 5) && (argc != 4))
{
  fprintf(stderr,
    "\nUsage: tensorDef <Def1> <Def2> <Def> [<topdim>] \n");
  fprintf(stderr,
    "  Computes the tensor product of the A-modules defined by the\n");
  fprintf(stderr,
    "  files <Def1> and <Def2>, and writes the result on the file <Def>\n");
  fprintf(stderr,
    "  It also prints on stdout the key expressing the generators of \n");
  fprintf(stderr,
    "  the tensor product in terms of the factors\n\n");
  fprintf(stderr,
    "  If the optional argument <topdim> is given, it takes the\n\n");
  fprintf(stderr,
    "  quotient by classes of degree higher than <topdim>.     \n\n");
  exit(1);
}

/* ------------------ open files --------------------------------------- */

if ((In1 = fopen(argv[1],"r")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to read\n",argv[1]);
  exit(1);
}

if ((In2 = fopen(argv[2],"r")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to read\n",argv[2]);
  exit(1);
}

if ((Out = fopen(argv[3],"w")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to write\n",argv[3]);
  exit(1);
}

/* ------------------ Now get headers  ------------------------------ */

fscanf(In1,"%d",&dim1);
fscanf(In2,"%d",&dim2);

deg1 = (int *)malloc(dim1*sizeof(int));
deg2 = (int *)malloc(dim2*sizeof(int));

for (i=0; i < dim1; i++)
  fscanf(In1,"%d",&deg1[i]);
for (i=0; i < dim2; i++)
  fscanf(In2,"%d",&deg2[i]);

for (mindeg1=maxdeg1=deg1[0],g=1; g<dim1; g++)
   { if (maxdeg1 < deg1[g]) maxdeg1 = deg1[g];
     if (mindeg1 > deg1[g]) mindeg1 = deg1[g];
     if (deg1[g-1] > deg1[g]) 
     {
       fprintf(stderr,"\nFirst module %s not in ascending order\n",argv[1]);
       exit(1);
     }
   }

for (mindeg2=maxdeg2=deg2[0],g=1; g<dim2; g++)
   { if (maxdeg2 < deg2[g]) maxdeg2 = deg2[g];
     if (mindeg2 > deg2[g]) mindeg2 = deg2[g];
     if (deg2[g-1] > deg2[g]) 
     {
       fprintf(stderr,"\nSecond module %s not in ascending order\n",argv[2]);
       exit(1);
     }
   }

/* ------------------ Make initial product -------------------------- */

mindeg = mindeg1 + mindeg2;
if (argc == 5) sscanf(argv[4],"%d",&maxdeg);
else maxdeg = maxdeg1 + maxdeg2;


first1 = (int *)malloc((maxdeg1-mindeg1+1)*sizeof(int));
second1 = (int *)malloc((maxdeg1-mindeg1+1)*sizeof(int));
first1 -= mindeg1;
second1 -= mindeg1;
for (i=mindeg1; i <= maxdeg1; i++) first1[i] = -1;
for (i=0; i < dim1; i++)
  second1[deg1[i]] = i;
for (i=dim1-1; i >= 0; i--)
  first1[deg1[i]] = i;

first2 = (int *)malloc((maxdeg2-mindeg2+1)*sizeof(int));
second2 = (int *)malloc((maxdeg2-mindeg2+1)*sizeof(int));
first2 -= mindeg2;
second2 -= mindeg2;
for (i=mindeg2; i <= maxdeg2; i++) first2[i] = -1;
for (i=0; i < dim2; i++)
  second2[deg2[i]] = i;
for (i=dim2-1; i >= 0; i--)
  first2[deg2[i]] = i;

for (dim=0, d=mindeg; d <= maxdeg; d++)
{ if (mindeg1 < d - maxdeg2) lo = d - maxdeg2;
  else lo = mindeg1;
  if (maxdeg1 > d - mindeg2) hi = d - mindeg2;
  else hi = maxdeg1;
  for (d1=lo; d1 <= hi; d1++)
  if ((first1[d1] >= 0) && (first2[d-d1] >= 0))
    dim += (second1[d1]-first1[d1] + 1)*(second2[d-d1]-first2[d-d1]+1);
}

deg = (int *)malloc(dim*sizeof(int));
first = (int *)malloc(dim*sizeof(int));
second = (int *)malloc(dim*sizeof(int));
product = (int **)malloc(dim1*sizeof(int *));
for (i=0; i < dim1; i++)
  product[i] = (int *)malloc(dim2*sizeof(int));


fprintf(Out,"%d\n",dim);

for (g=0, d=mindeg; d <= maxdeg; d++)
{ if (mindeg1 < d - maxdeg2) lo = d - maxdeg2;
  else lo = mindeg1;
  if (maxdeg1 > d - mindeg2) hi = d - mindeg2;
  else hi = maxdeg1;
  for (d1=lo; d1 <= hi; d1++)
  if ((first1[d1] >= 0) && (first2[d-d1] >= 0))
  { for (x1=first1[d1]; x1 <= second1[d1]; x1++)
    for (x2=first2[d-d1]; x2 <= second2[d-d1]; x2++)
    {  first[g] =x1; 
       second[g] = x2;
       deg[g] = d;
       product[x1][x2] = g;
       printf("%d %d %d\n",g, x1, x2);
       if (!(g&15)) fprintf(Out,"\n");
       fprintf(Out,"%d ",d);
       g++;
    }
  }
}

fprintf(Out,"\n\n");

if (g != dim)
{  fprintf(stderr,"\nInconsistent count of generators\n\n");
   exit(1);
}


/* ----------- Now read in the action on the two factors   ----------- */

sq1 = read_sqs(In1,dim1,deg1,argv[1]);
sq2 = read_sqs(In2,dim2,deg2,argv[2]);

/* -------------------- and tensor them   --------------------------- */

  sq=(module_element ***)malloc(dim*sizeof(module_element **));
  for (g=0;g<dim;g++)
  {
    sq[g]=(module_element **)malloc(
	 (1+maxdeg - deg[g])*sizeof(module_element *));
    sq[g][0] = (module_element *)malloc(sizeof(module_element));
    sq[g][0]->first = g;
    sq[g][0]->next = NULL;
    for(i=1;i<=maxdeg - deg[g];i++)
      sq[g][i]=NULL;
  }
    		/* sq[g][op]'s are initially NULL 	*/
		/* except Sq(0) is the identity op  		*/


for (g=0; g< dim; g++)
for (n=1; n <= maxdeg - deg[g]; n++)
{
for (i=0; i<= n; i++)
if ((i + deg1[first[g]] <= maxdeg1) && ((n-i+deg2[second[g]]) <= maxdeg2))
for (g1=sq1[first[g]][i]; g1 != NULL; g1 = g1->next)
for (g2=sq2[second[g]][n-i]; g2 != NULL; g2 = g2->next)
sq[g][n] = insert_gen(product[g1->first][g2->first],sq[g][n]);

for (s=sq[g][n], l=0; s!= NULL; s=s->next, l++);
if (l != 0)
{
 fprintf(Out,"%d %d %d",g,n,l); 
 for (s=sq[g][n]; s!= NULL; s=s->next)
    fprintf(Out," %d",s->first);
 fprintf(Out,"\n");
}
}

/* --------------------------- Done   ------------------------------ */

fclose(Out);
return(0);
   
}

