/* -------------------------------------------------------------------	*/
/* Copyright (C) 2008 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.8								*/
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */


#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <tensor.h>
#include <steen.h>


typedef struct LIST { int item;
               struct LIST *next; }  LIST;


main(argc,argv) int argc; char *argv[];

{

int i,s,t,s0,g,g0,g1,n;
int notempty,count,maxstem, minstem;
int *Maxt;
LIST ***Num;
Tensor_shape  *shape;
FILE *Maxtfile, *Lines;
LIST ****his;                      /* his = plural of h sub i */
LIST *temp, *it;

char message[100];

if (argc != 2)
{  printf("\nUsage:  vsumm <title>\n");
   printf("\n  Reads Shape, Maxt and lines and prints a stem by stem summary of");
   printf("\n  the resolution on stdout, in LaTex, using <title> as the title.\n\n");
   exit(0);
}


/* ------------------ get shape  ----------------------------------- */

shape = read_shape(-1);

maxstem = minstem = shape->deg[0][0];
for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
 { if (maxstem < shape->deg[s][g]-s) maxstem = shape->deg[s][g]-s;
   if (minstem > shape->deg[s][g]-s) minstem = shape->deg[s][g]-s;
 }

Num = (LIST ***) malloc(((++maxstem)-minstem)*sizeof(LIST **));
Num -= minstem;
		/*      Now Num[minstem] through Num[maxstem-1] exist	*/
for (n=minstem; n < maxstem; n++)
{  Num[n] = (LIST  **) malloc((1+shape->s)*sizeof(LIST *));
   for (s=0; s<= shape->s; s++) Num[n][s] = NULL;
}

for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
{
   n = shape->deg[s][g]-s;
   temp = (LIST *) malloc(sizeof(LIST));
   temp->item = g;
   temp->next = Num[n][s];
   Num[n][s] = temp;
} 


/* ------------------ get Maxt  ----------------------------------- */

if ((Maxtfile = fopen("Maxt","r")) == NULL)
{  sprintf(message,"\nCannot open Maxt.  Run report \n");
   terminate(message,BAD_WORKFILE);
}
Maxt = (int *) malloc((1+shape->s)*sizeof(int));
for (s=0; s<=shape->s; s++)
  Maxt[s] = shape->deg[0][0];
while (EOF != fscanf(Maxtfile,"%d %d",&s,&t))
{
  if ((0 <= s) && (s <= shape->s))  Maxt[s] = t;
}
fclose(Maxtfile);


/* ------------------ Now create his data structure  ------------------ */

his = (LIST ****) malloc((1+shape->s)*sizeof(LIST ***));
for (s=0; s<=shape->s; s++)
{ his[s] = (LIST ***) malloc((shape->numgen[s])*sizeof(LIST **));
  for (g=0; g<shape->numgen[s]; g++)
  { his[s][g] = (LIST **) malloc((1+N_ALG)*sizeof(LIST *));
    for (i=0; i<= N_ALG; i++)
      his[s][g][i] = NULL;
}
}


/* ------------------ Now read his  ----------------------------------- */


if ((Lines = fopen("lines","r")) == NULL)
{  sprintf(message,"\nCannot open lines file\n");
   terminate(message,BAD_WORKFILE);
}

while (EOF != fscanf(Lines,"%d %d %d %d %d",&s,&g,&s0,&g0,&count))
{ 
  for (i=0; i<count; i++)
  { fscanf(Lines,"%d",&g1);
    temp = (LIST *) malloc(sizeof(LIST));
    temp->item = g;
    temp->next = his[s-1][g1][g0];
    his[s-1][g1][g0] = temp;

  }
}

fclose(Lines);

/* -------------------- Now print out all stems -------------- */
printf("\\documentclass{article}\n");
printf("\\usepackage{fullpage}\n");
printf("\\usepackage{longtable}\n");
printf("\\begin{document}\n");
printf("\\title{%s}\n",argv[1]);
printf("\\maketitle\n");
printf("Notes:\n\\begin{enumerate}\n");
printf(" \\item  Stem refers to the geometric total degree $n = t-s$,\n");
printf("   where $t$ is the\n");
printf("  internal degree and $s$ is the homological degree (or `filtration').\n");
printf(" \\item  If a stem is not printed, there are no elements in that stem.\n");
printf(" \\item  The notation $s_g$ refers to generator number $g$ in\n");
printf("  filtration $s$.\n");
printf(" \\item  Dashes (--) are used to indicate that an $h_i$ multiplen");
printf("  is beyond the range which has been calculated.\n");
printf("\\end{enumerate}\n\n");



for (n=minstem; n<maxstem; n++)
{
  for (notempty=0,s=0; s <= shape->s; s++)
    { if (Num[n][s] != NULL) notempty =1; }
  if (notempty == 1)
{
  printf("\\begin{longtable}{|rrr|r|r|r|r|r|r|r|r|}\n");
  printf("\\caption{Stem %d}\\\\\n",n);
  printf("\\hline\n");
  printf(
   "n&s&g &$h_0$ & $h_1$ & $h_2$ & $h_3$ & $h_4$ & $h_5$ & $h_6$ & $h_7$ \\\\\n");
  printf("\\hline\n");
  printf("\\endhead\n");
  printf("\\hline\n");
  
  

  for (s=0; s<=shape->s; s++)
    if (Num[n][s] != NULL)
  {
    for (temp=Num[n][s]; temp != NULL; temp=temp->next)
    {
       g = temp->item;
       printf("%d & %d & $%d_{%d}$ ",n,s,s,g);
       for (i=0; i<=N_ALG; i++)
         {
           printf("& ");
           if ((s+1 > shape->s) || (n+s+(1<<i) > Maxt[s+1])) 
             printf(" -- ");
           else 
           for (it=his[s][g][i]; it != NULL; it = it->next)
           {
             printf("$%d_{%d}$ ",s+1,it->item);
             if (it->next != NULL)
                printf("+ ");
           }
         }
       printf("\\\\\n");
    }
    printf("\\hline\n");
  }

  printf("\\end{longtable}\n\n");
}
}

printf("\\end{document}\n");
   
return(0);

}

